<?php

namespace Drupal\block_class\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\block\Entity\Block;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Component\Serialization\Json;

/**
 * Controller routines for help routes.
 */
class BlockClassController extends ControllerBase {

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The extension list module.
   *
   * @var \Drupal\Core\Extension\ExtensionList
   */
  protected $extensionListModule;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Creates a new HelpController.
   */
  public function __construct(RouteMatchInterface $route_match, ExtensionList $extension_list_module, ConfigFactoryInterface $config_factory) {
    $this->routeMatch = $route_match;
    $this->extensionListModule = $extension_list_module;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('extension.list.module'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function index() {
    $build = [];
    $projectMachineName = 'block_class';

    $projectName = $this->moduleHandler()->getName($projectMachineName);

    $build['#title'] = 'Block Class Help';

    $helperMarkup = $this->moduleHandler()->invoke($projectMachineName, 'help', [
      "help.page.$projectMachineName",
      $this->routeMatch,
    ]);

    if (!is_array($helperMarkup)) {
      $helperMarkup = ['#markup' => $helperMarkup];
    }

    $build['top'] = $helperMarkup;

    // Only print list of administration pages if the project in question has
    // any such pages associated with it.
    $adminTasks = system_get_module_admin_tasks($projectMachineName, $this->extensionListModule->getExtensionInfo($projectMachineName));

    if (empty($adminTasks)) {
      return $build;
    }

    $links = [];

    foreach ($adminTasks as $adminTask) {

      $link['url'] = $adminTask['url'];

      $link['title'] = $adminTask['title'];

      if ($link['url']->getRouteName() === 'block_class.settings') {
        $link['title'] = 'Block Class Settings';
      }

      $links[] = $link;
    }

    $build['links'] = [
      '#theme' => 'links__help',
      '#heading' => [
        'level' => 'h3',
        'text' => $this->t('@project_name administration pages', ['@project_name' => $projectName]),
      ],
      '#links' => $links,
    ];

    return $build;
  }

  /**
   * Method to show the block list.
   */
  public function blockList() {

    // Get config object.
    $config = $this->configFactory->getEditable('block_class.settings');

    $table = '<table>';
    $table .= '<thead>';
    $table .= '<tr>';
    $table .= '<th>' . $this->t('Block') . '</th>';
    $table .= '<th>' . $this->t('Class') . '</th>';
    $table .= '<th>' . $this->t('Attributes') . '</th>';
    $table .= '<th>' . $this->t('Edit') . '</th>';
    $table .= '<th>' . $this->t('Delete') . '</th>';
    $table .= '</tr>';
    $table .= '</thead>';
    $table .= '<tbody>';

    // Load blocks. Todo: We'll implements DI here @codingStandardsIgnoreLine
    $blocks = Block::loadMultiple();

    foreach ($blocks as $block) {

      if (empty($block->getThirdPartySetting('block_class', 'classes')) && empty($block->getThirdPartySetting('block_class', 'attributes'))) {
        continue;
      }

      // Get the block classes configured.
      $block_class = $block->getThirdPartySetting('block_class', 'classes');

      // Get the attributes.
      $attributes = $block->getThirdPartySetting('block_class', 'attributes');

      // If classes is empty, and there are attributes but the flag to enable
      // attributes is off, skip.
      if (empty($block_class) && !empty($attributes) && empty($config->get('enable_attributes'))) {
        continue;
      }

      // Put one attribute per line.
      $attributes = str_replace(PHP_EOL, '<br>', $attributes);

      $table .= '<tr>';
      $table .= '<td>' . '<a href="/admin/structure/block/manage/' . $block->id() . '">' . $block->label() . '</a></td>';
      $table .= '<td>' . $block_class . '</td>';
      $table .= '<td>' . $attributes . '</td>';
      $table .= '<td>' . '<a href="/admin/structure/block/manage/' . $block->id() . '">' . $this->t('Edit') . '</a></td>';
      $table .= '<td>' . '<a href="/admin/config/content/block-class/delete/' . $block->id() . '">' . $this->t('Delete') . '</a></td>';
      $table .= '</tr>';
    }

    $table .= '</tbody>';
    $table .= '</table>';

    $build = [
      '#markup' => $table,
    ];

    return $build;

  }

  /**
   * Handle Auto Complete.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Json Response.
   */
  public function handleAutocomplete(Request $request) {

    $block_classes_stored = [];

    $config = $this->configFactory->getEditable('block_class.settings');

    // Get config object.
    if (!empty($config->get('block_classes_stored'))) {
      $block_classes_stored = $config->get('block_classes_stored');
    }

    // Get the array.
    $block_classes_stored = Json::decode($block_classes_stored);

    // Return in JSON Response.
    return new JsonResponse($block_classes_stored);
  }

}
