<?php

namespace Drupal\block_class\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;

/**
 * Form for Block Class Settings.
 */
class BlockClassSettingsForm extends ConfigFormBase {

  use StringTranslationTrait;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The extension list module.
   *
   * @var \Drupal\Core\Extension\ExtensionList
   */
  protected $extensionListModule;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Creates the construct.
   */
  public function __construct(RouteMatchInterface $route_match, ExtensionList $extension_list_module, ConfigFactoryInterface $config_factory, MessengerInterface $messenger) {
    $this->routeMatch = $route_match;
    $this->extensionListModule = $extension_list_module;
    $this->configFactory = $config_factory;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('extension.list.module'),
      $container->get('config.factory'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'block_class_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'block_class.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('block_class.settings');

    // Default value.
    $default_case = 'standard';

    // Verify if there is a value in the database.
    if (!empty($config->get('default_case'))) {
      $default_case = $config->get('default_case');
    }

    $form['global_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Global Settings'),
      '#open' => TRUE,
    ];

    $form['global_settings']['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General'),
      '#open' => TRUE,
    ];

    $form['global_settings']['general']['default_case'] = [
      '#title' => $this->t("Default Case"),
      '#type' => 'select',
      '#description' => $this->t('If you select "Uppercase and Lowercase" but cases will be accepted. If you select "Uppercase" all classes will be added using uppercase and if you select "Lowercase" all classes added will be added using lowercase.'),
      '#options' => [
        'standard' => $this->t('Uppercase and Lowercase (Standard)'),
        'uppercase' => $this->t('Uppercase'),
        'lowercase' => $this->t('Lowercase'),
      ],
      '#default_value' => $default_case,
    ];

    $form['global_settings']['class'] = [
      '#type' => 'details',
      '#title' => $this->t('Class'),
      '#open' => TRUE,
    ];

    $field_type = 'textfield';

    if (!empty($config->get('field_type'))) {
      $field_type = $config->get('field_type');
    }

    $form['global_settings']['class']['field_type'] = [
      '#title' => $this->t("Field Type"),
      '#type' => 'select',
      '#options' => [
        'textfield' => $this->t('textfield'),
        'textarea' => $this->t('textarea'),
      ],
      '#default_value' => $field_type,
    ];

    $maxlength_block_class_field = 255;

    if (!empty($config->get('maxlength_block_class_field'))) {
      $maxlength_block_class_field = $config->get('maxlength_block_class_field');
    }

    $form['global_settings']['class']['maxlength_block_class_field'] = [
      '#title' => $this->t("Maxlength"),
      '#type' => 'number',
      '#description' => $this->t('This will be the default maxlength value for the "maxlength" field. The default is 255.'),
      '#default_value' => $maxlength_block_class_field,
    ];

    $form['global_settings']['class']['allow_only_letters_and_numbers'] = [
      '#title' => $this->t("Allow only letters, numbers, hyphen and underline"),
      '#type' => 'checkbox',
      '#description' => $this->t('If selected the classes will allows only letters, numbers, hyphen and underlines.'),
      '#default_value' => $config->get('allow_only_letters_and_numbers'),
    ];

    $form['global_settings']['attributes'] = [
      '#type' => 'details',
      '#title' => $this->t('Attributes'),
      '#open' => TRUE,
    ];

    $form['global_settings']['attributes']['enable_attributes'] = [
      '#title' => $this->t("Enable attributes"),
      '#type' => 'checkbox',
      '#default_value' => $config->get('enable_attributes'),
    ];

    $field_type_attributes = 'textarea';

    if (!empty($config->get('field_type_attributes'))) {
      $field_type_attributes = $config->get('field_type_attributes');
    }

    $form['global_settings']['attributes']['field_type_attributes'] = [
      '#title' => $this->t("Field Type"),
      '#type' => 'select',
      '#options' => [
        'textfield' => $this->t('textfield'),
        'textarea' => $this->t('textarea'),
      ],
      '#default_value' => $field_type_attributes,
      '#states' => [
        'visible' => [
          ':input[name="enable_attributes"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $maxlength_attributes = FALSE;

    if (!empty($config->get('maxlength_attributes'))) {
      $maxlength_attributes = $config->get('maxlength_attributes');
    }

    $form['global_settings']['attributes']['maxlength_attributes'] = [
      '#title' => $this->t("Maxlength"),
      '#type' => 'number',
      '#description' => $this->t('This will be the default maxlength value for the attributes field'),
      '#default_value' => $maxlength_attributes,
      '#states' => [
        'visible' => [
          ':input[name="enable_attributes"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['global_settings']['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced'),
      '#open' => FALSE,
    ];

    $form['global_settings']['advanced']['filter_html_clean_css_identifier'] = [
      '#title' => $this->t("Filter to HTML Clean CSS Identifier"),
      '#type' => 'textarea',
      '#description' => $this->t('You can use this field to insert the configuration of <a href="https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Component%21Utility%21Html.php/function/Html%3A%3AcleanCssIdentifier/">Filter to HTML Clean CSS Identifier</a>. You can insert to replace special chars in the class. Use key|value format, and one per line. E.g.<br>#|-<br>%|-'),
      '#default_value' => $config->get('filter_html_clean_css_identifier'),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;

  }

  /**
   * Validate the maxlength field.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    parent::validateForm($form, $form_state);

    // Get the maxlength_block_class_field field.
    $maxlength_block_class_field = $form_state->getValue('maxlength_block_class_field');

    // Verify if is a positive value.
    if ($maxlength_block_class_field < 1) {
      $form_state->setErrorByName('maxlength_block_class_field', $this->t('The maxlength should be an positive value'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // Get the config object.
    $config = $this->config('block_class.settings');

    $previous_default_case = $config->get('default_case');

    // Get the default case.
    $default_case = $form_state->getValue('default_case');

    // Get the field type.
    $field_type = $form_state->getValue('field_type');

    $field_type_attributes = $form_state->getValue('field_type_attributes');

    // Get the maxlength_block_class_field field.
    $maxlength_block_class_field = $form_state->getValue('maxlength_block_class_field');

    $maxlength_attributes = $form_state->getValue('maxlength_attributes');

    if (empty($form_state->getValue('maxlength_attributes'))) {
      $maxlength_attributes = 10000;
    }

    $allow_only_letters_and_numbers = $form_state->getValue('allow_only_letters_and_numbers');

    $enable_attributes = $form_state->getValue('enable_attributes');

    $filter_html_clean_css_identifier = $form_state->getValue('filter_html_clean_css_identifier');

    // Set the values.
    $config->set('default_case', $default_case);
    $config->set('field_type', $field_type);
    $config->set('field_type_attributes', $field_type_attributes);
    $config->set('maxlength_block_class_field', $maxlength_block_class_field);
    $config->set('allow_only_letters_and_numbers', $allow_only_letters_and_numbers);
    $config->set('enable_attributes', $enable_attributes);
    $config->set('maxlength_attributes', $maxlength_attributes);
    $config->set('filter_html_clean_css_identifier', $filter_html_clean_css_identifier);

    // Save that.
    $config->save();

    parent::submitForm($form, $form_state);

    if ($previous_default_case != $default_case) {

      $this->messenger->addStatus($this->t('Now you are using @default_case@ as a default case. If you want to convert all classes stored, feel free run the <a href="/admin/config/content/block-class/bulk-operations">Bulk Update</a> to convert all to @default_case@', [
        '@default_case@' => $default_case,
      ]));

    }

  }

}
