<?php

namespace Drupal\block_class\Service;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\block\Entity\Block;
use Drupal\Component\Utility\Html;
use Drupal\block_class\Constants\BlockClassConstants;
use Drupal\Component\Serialization\Json;

/**
 * Block Class Service Class.
 */
class BlockClassHelperService {

  use StringTranslationTrait;

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Path Matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The user current.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * Drupal\Core\Entity\EntityRepositoryInterface service.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Construct of Block Class service.
   */
  public function __construct(LanguageManagerInterface $language_manager, EntityTypeManagerInterface $entity_manager, ConfigFactoryInterface $config_factory, Connection $database, RequestStack $request_stack, PathMatcherInterface $path_matcher, UuidInterface $uuid_service, AccountProxyInterface $current_user, AliasManagerInterface $alias_manager, CurrentPathStack $current_path, EntityRepositoryInterface $entityRepository) {
    $this->languageManager = $language_manager;
    $this->entityTypeManager = $entity_manager;
    $this->pathMatcher = $path_matcher;
    $this->request = $request_stack->getCurrentRequest();
    $this->configFactory = $config_factory;
    $this->database = $database;
    $this->uuidService = $uuid_service;
    $this->currentUser = $current_user;
    $this->aliasManager = $alias_manager;
    $this->currentPath = $current_path;
    $this->entityRepository = $entityRepository;
  }

  /**
   * Method to do the presave block.
   */
  public function blockClassPreSave(&$entity) {

    // If there is no class, unset the Third Party Setting.
    if (empty($entity->getThirdPartySetting('block_class', 'classes'))) {
      $entity->unsetThirdPartySetting('block_class', 'classes');
    }

    // If there is no attributes, unset the Third Party Setting.
    if (empty($entity->getThirdPartySetting('block_class', 'attributes'))) {
      $entity->unsetThirdPartySetting('block_class', 'attributes');
    }

    // Get the config object.
    $config = $this->configFactory->getEditable('block_class.settings');

    // Get the default case on settings.
    $default_case = $config->get('default_case', 'standard');

    // Get the block class.
    $block_classes = $entity->getThirdPartySetting('block_class', 'classes');

    switch ($default_case) {

      case 'uppercase':

        $block_classes = strtoupper($block_classes);

        break;

      case 'lowercase':

        $block_classes = strtolower($block_classes);

        break;

    }

    // Set the Third Party Settings.
    $entity->setThirdPartySetting('block_class', 'classes', $block_classes);

    // Get the config object.
    $config = $this->configFactory->getEditable('block_class.settings');

    // Get the current classes stored.
    $block_classes_stored = $config->get('block_classes_stored');

    // Get the array from JSON.
    $block_classes_stored = Json::decode($block_classes_stored);

    // Verify if is empty.
    if (empty($block_classes_stored)) {
      $block_classes_stored = [];
    }

    // Get the current class and export to array.
    $current_block_classes = explode(' ', $block_classes);

    // Use the key the same as value.
    $current_block_classes = array_combine($current_block_classes, $current_block_classes);

    // Merge with the current one.
    $block_classes_to_store = array_merge($current_block_classes, $current_block_classes);

    // Get as JSON.
    $block_classes_to_store = Json::encode($block_classes_to_store);

    // Store in the config.
    $config->set('block_classes_stored', $block_classes_to_store);

    // Save.
    $config->save();

  }

  /**
   * Method to redirect to Bulk Operations.
   */
  public function redirectToBulkOperations() {

    // Get path bulk operation.
    $bulk_operation_path = Url::fromRoute('block_class.bulk_operations')->toString();

    // Get response.
    $response = new RedirectResponse($bulk_operation_path);

    // Send to confirmation.
    $response->send();
    exit;

  }

  /**
   * Method to do the preprocess block.
   */
  public function blockClassPreprocessBlock(&$variables) {

    // Blocks coming from page manager widget does not have id. If there is no
    // Block ID, skip that.
    if (empty($variables['elements']['#id'])) {
      return;
    }

    // If there is no block with this ID, skip.
    if (empty(Block::load($variables['elements']['#id']))) {
      return;
    }

    // Load the block by ID.
    $block = Block::load($variables['elements']['#id']);

    // Add attributes on block.
    $this->addAttributesOnBlock($block, $variables);

    // Add classes on block.
    $this->addClassesOnBlock($block, $variables);

  }

  /**
   * Method to add attributes on block.
   */
  public function addAttributesOnBlock(&$block, &$variables) {

    // Get config object.
    $config = \Drupal::config('block_class.settings');

    // If attributes isn't enabled, skip.
    if (empty($config->get('enable_attributes'))) {
      return FALSE;
    }

    // If there is no attributes on block, skip.
    if (empty($block->getThirdPartySetting('block_class', 'attributes'))) {
      return FALSE;
    }

    $attributes = $block->getThirdPartySetting('block_class', 'attributes');

    $attributes = explode(PHP_EOL, $attributes);

    foreach ($attributes as $attribute) {

      $attribute = explode('|', $attribute);

      $attribute_key = trim($attribute[0]);

      $attribute_value = trim($attribute[1]);

      // Insert the attributes.
      $variables['attributes'][$attribute_key][] = $attribute_value;
    }
  }

  /**
   * Method to add classes on block.
   */
  public function addClassesOnBlock(&$block, &$variables) {

    // Verify if the current block has Third Party Settings with classes.
    $classes = $block->getThirdPartySetting('block_class', 'classes');

    if (empty($classes)) {
      return FALSE;
    }

    // Get all classes if exists.
    $classes_array = explode(' ', $classes);

    // Get config object.
    $config = \Drupal::config('block_class.settings');

    $filter_css_identifier = [];

    if (!empty($config->get('filter_html_clean_css_identifier'))) {

      $filter_clean_css_identifier = $config->get('filter_html_clean_css_identifier');

      $filter_clean_css_identifier = explode(PHP_EOL, $filter_clean_css_identifier);

      foreach ($filter_clean_css_identifier as $filter) {

        $filter = explode('|', $filter);

        $origin_char = trim($filter[0]);
        $target_char = trim($filter[1]);

        $filter_css_identifier[$origin_char] = $target_char;

      }

    }

    // Add all classes.
    foreach ($classes_array as $class) {
      $variables['attributes']['class'][] = Html::cleanCssIdentifier($class, $filter_css_identifier);
    }
  }

  /**
   * Method to do a form alter.
   */
  public function blockClassFormAlter(&$form, &$form_state) {

    // If the user don't have permission, skip that.
    if (!\Drupal::currentUser()->hasPermission(BlockClassConstants::BLOCK_CLASS_PERMISSION)) {
      return;
    }

    $form_object = $form_state->getFormObject();

    if ($form_object instanceof EntityFormInterface) {

      /** @var \Drupal\block\BlockInterface $block */
      $block = $form_object->getEntity();

      // This will automatically be saved in the third party settings.
      $form['third_party_settings']['#tree'] = TRUE;

      // Default field type.
      $field_type = 'textfield';

      // Default value for maxlength.
      $maxlength_block_class_field = 255;

      // Get config object.
      $config = \Drupal::config('block_class.settings');

      // Get the field type if exists.
      if (!empty($config->get('field_type'))) {
        $field_type = $config->get('field_type');
      }

      // Get maxlength if exists.
      if (!empty($config->get('maxlength_block_class_field'))) {
        $maxlength_block_class_field = $config->get('maxlength_block_class_field');
      }

      $form['third_party_settings']['block_class']['classes'] = [
        '#type' => $field_type,
        '#title' => $this->t('CSS class(es)'),
        '#description' => $this->t('Customize the styling of this block by adding CSS classes. Separate multiple classes by spaces. The maxlength configured is @maxlength_block_class_field@. If necessary you can update it <a href="/admin/config/content/block-class/settings">here</a>', [
          '@maxlength_block_class_field@' => $maxlength_block_class_field,
        ]),
        '#default_value' => $block->getThirdPartySetting('block_class', 'classes'),
        '#maxlength' => $maxlength_block_class_field,
      ];

      $form['third_party_settings']['block_class']['classes']['#attributes']['class'][] = 'block-class-class';

      if (!empty($config->get('enable_attributes'))) {

        // Default field type.
        $field_type_attributes = 'textarea';

        // Get the field type if exists.
        if (!empty($config->get('field_type_attributes'))) {
          $field_type_attributes = $config->get('field_type_attributes');
        }

        $form['third_party_settings']['block_class']['attributes'] = [
          '#type' => $field_type_attributes,
          '#title' => $this->t('Attributes'),
          '#description' => $this->t('Here you can insert any attributes, use one per line. For example: data-block-type|info'),
          '#default_value' => $block->getThirdPartySetting('block_class', 'attributes'),
        ];

        $form['third_party_settings']['block_class']['attributes']['#attributes']['class'][] = 'block-class-attributes';

        // Get maxlength if exists.
        if (!empty($config->get('maxlength_attributes'))) {
          $form['third_party_settings']['block_class']['attributes']['#maxlength'] = $config->get('maxlength_attributes');
        }
      }

      if (empty($block->getThirdPartySetting('block_class', 'classes'))) {
        $form['third_party_settings']['block_class']['classes']['#autocomplete_route_name'] = 'block_class.autocomplete';
      }
    }

    $form['#validate'][] = 'block_class_form_block_form_validate';

    $form['#attached']['library'][] = 'block_class/block-class';

    // Get the default case on settings.
    $default_case = $config->get('default_case', 'standard');

    // Put the "default case" in the Drupal settings to be used in the JS.
    $form['#attached']['drupalSettings']['block_class']['default_case'] = $default_case;

  }

  /**
   * Method to do a form validate.
   */
  public function blockClassFormValidate(&$form, &$form_state) {

    // Get config object.
    $config = \Drupal::config('block_class.settings');

    // Validate class.
    $this->validateClass($form, $form_state, $config);

    // Validate attributes.
    $this->validateAttributes($form, $form_state, $config);

  }

  /**
   * Method to validate attributes.
   */
  public function validateAttributes(&$form, &$form_state, $config) {

    if (empty($config->get('enable_attributes'))) {
      return FALSE;
    }

    $third_party_settings = $form_state->getValue('third_party_settings');

    if (empty($third_party_settings['block_class']['attributes'])) {
      return FALSE;
    }

    $attributes = $third_party_settings['block_class']['attributes'];

    $pipe_found = strpos($attributes, '|');

    if ($pipe_found === FALSE) {

      $form_state->setErrorByName('third_party_settings][block_class][attributes', $this->t("You need to insert a pipe on attributes field"));

      return FALSE;

    }

    // Prevent attribute id.
    if (str_contains($attributes, 'id|')) {

      $form_state->setErrorByName('third_party_settings][block_class][attributes', $this->t("You can't the attribute id"));

      return FALSE;

    }

    // Prevent attribute class.
    if (str_contains($attributes, 'class|')) {

      $form_state->setErrorByName('third_party_settings][block_class][attributes', $this->t("You can't use class. Use the field class instead"));

      return FALSE;

    }

    // If it's only one line, skip.
    if (!str_contains($attributes, PHP_EOL)) {
      return FALSE;
    }

    $attributes = explode(PHP_EOL, $attributes);

    foreach ($attributes as $attribute) {

      if (!str_contains($attribute, '|')) {

        $form_state->setErrorByName('third_party_settings][block_class][attributes', $this->t("You need to insert a pipe on attributes field"));

        return FALSE;

      }

    }

  }

  /**
   * Method to validate class.
   */
  public function validateClass(&$form, &$form_state, $config) {

    // If there is a settings to allow only letters and numbers, validate this.
    if (empty($config->get('allow_only_letters_and_numbers'))) {
      return FALSE;
    }

    $third_party_settings = $form_state->getValue('third_party_settings');

    if (empty($third_party_settings['block_class']['classes'])) {
      return FALSE;
    }

    $classes = $third_party_settings['block_class']['classes'];

    if (preg_match('/[\'^£$%&*()}{@#~?><>,|=+¬]/', $classes)) {

      $form_state->setErrorByName('third_party_settings][block_class][classes', $this->t("In class is allowed only letters, numbers, hyphen and underline"));

      return FALSE;

    }

  }

}
